//Structure based on Class Example(W24) from Jenni Hutson
//Edited to fit the dataset and aesthetics by N.M.

import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;

//GUI things
boolean showBox = true;
boolean showGrid = true;
boolean showSet1=true;
boolean showSet2 = true;
boolean showItemsTitles = true;
boolean showCurves = true;


// Box dimensions
int boxWidth = 1600;
int boxHeight = 1200;
int boxDepth = 1000;

int startYear = 2021;
int endYear = 2023;

int startTime = 8*60;
int totalDaySpan = 6574;
int totalMinuteSpan = 720;
int totalDaysInYear = 365;

Table table;
Table table2;

ArrayList<Item> items = new ArrayList<Item>(); // Store the dataset items
ArrayList<Item> items2 = new ArrayList<Item>(); // Store the second dataset items

void setup() {
  size(1920, 1080, P3D);
   
  cam = new PeasyCam(this, 3000);
  
  cam.setMinimumDistance(200);
  cam.setMaximumDistance(15000);
  
  cp5 =new ControlP5(this);
   
  setGui();
  
  table = loadTable("ai_new_approach.csv", "header");
  for (TableRow row : table.rows()) {
    String title = row.getString("title");
    String itemtype = row.getString("itemtype");
    String checkoutDate = row.getString("CheckoutDate");
    int checkoutHour = row.getInt("CheckoutHour");
    int checkoutDuration = row.getInt("CheckoutDurationHours");
    float deweyClass = row.getFloat("deweyClass");
    Item newItem = new Item(title, itemtype, checkoutDate, checkoutHour, checkoutDuration, deweyClass);
    //newItem.display();   
    items.add(newItem);
    } 
    
  table2 = loadTable("textile_new_approach.csv", "header");
  for (TableRow row : table2.rows()) {
    String title = row.getString("title");
    String itemtype = row.getString("itemtype");
    String checkoutDate = row.getString("CheckoutDate");
    int checkoutHour = row.getInt("CheckoutHour");
    int checkoutDuration = row.getInt("CheckoutDurationHours");
    float deweyClass = row.getFloat("deweyClass");
    Item newItem = new Item(title, itemtype, checkoutDate, checkoutHour, checkoutDuration, deweyClass);
    items2.add(newItem);
  }
 }

void draw() {
  background(0); 
  
  if(showItemsTitles) {
  if(showSet1) {
    drawItemsAndTitles(items2, color(140, 90, 210)); 
  } 
  if(showSet2) {
    drawItemsAndTitles(items, color(180,215,205));
  }
}

if(showCurves) {
  if(showSet1) {     
    drawCurves(items2, color(80,0,185), color(170,75,85)); 
  }
  if(showSet2) {
    drawCurves(items, color(100,210,155), color(110, 135, 255)); 
  }
}

     
  //Box   
  if (showBox) {
  stroke(255);
  noFill();
  strokeWeight(1);
  box(boxWidth, boxDepth, boxHeight); 
  }
  //Grid
  if (showGrid){ 
  drawGrid(boxWidth, boxDepth, 36,12,12); 
  }
  drawLabels();
  gui();
  
}

void drawItemsAndTitles(ArrayList<Item> items, color strokeColor) {
  for (int i = 0; i < items.size(); i++) {
    Item item = items.get(i);
    float x = map(parseYear(item.checkoutDate), 0, 1095, -boxWidth / 2, boxWidth / 2);
    float y = log(item.checkoutDuration +1);
    y = map(y, 0, log(maxCheckoutDuration() +1), -boxDepth / 2, boxDepth / 2);
    float z = map(item.checkoutHour, 8, 20, -boxHeight / 2, boxHeight / 2);

    // Draw each item here
    pushMatrix();
    translate(x, y, z);
    stroke(strokeColor);
    strokeWeight(6);
    noFill();
    point(0, 0, 0);  
    
     // Check if the mouse is close to this item
    float screenX = screenX(0, 0, 0);
    float screenY = screenY(0, 0, 0);
    popMatrix();
        
    if (dist(mouseX, mouseY, screenX, screenY) < 10) {
      cp5.get(Textlabel.class, "dynamicTitle").setText(item.title.toUpperCase());
      cp5.get(Textlabel.class, "dynamicType").setText(item.itemtype.toString());
      cp5.get(Textlabel.class, "dynamicDate").setText(item.checkoutDate.toString());
      
      //break; //this is a cool effect
    }
  }
}

void drawCurves(ArrayList<Item> items, color startColor, color endColor) {
 
  strokeWeight(1);
  noFill();
  
  beginShape();
  for (int i = 0; i < items.size(); i++) {
    Item item = items.get(i);

    // Calculate position
    float x = map(parseYear(item.checkoutDate), 0, 1095, -boxWidth / 2, boxWidth / 2);
    float rawY = log(item.checkoutDuration + 1);
    float y = map(rawY, 0, log(maxCheckoutDuration() + 1), -boxDepth / 2, boxDepth / 2);
    float z = map(item.checkoutHour, 8, 20, -boxHeight / 2, boxHeight / 2);
    
    float colorPercent = map(i,0,items.size(), 0.0, 1.0);
    color currColor = lerpColor (startColor, endColor, colorPercent);
    stroke(currColor);
    
    // Processing requires at least 4 points to start rendering the curve
    if (i == 0 || i == items.size() - 1) {
      curveVertex(x, y, z); // Duplicate the first and last points for the curve to "reach" them
    }
    curveVertex(x, y, z);
  }
  endShape();
}
