void drawGrid(float width, float depth, int widthLines, int depthLines, int heightLines) {
  stroke(255); 
  strokeWeight(0.5); 
  
  beginShape(LINES);
  for (int i = 0; i <= widthLines; i++) {
    float posX = map(i, 0, widthLines, -width / 2, width / 2);
    // Lines parallel to Z-axis
    vertex(posX, boxHeight / 2, -depth / 2);
    vertex(posX, boxHeight / 2, depth / 2);
  }
  for (int i = 0; i <= depthLines; i++) {
    float posZ = map(i, 0, depthLines, -depth / 2, depth / 2);
    // Lines parallel to X-axis
    vertex(-width / 2, -boxHeight / 2, posZ);
    vertex(width / 2, -boxHeight / 2, posZ);
  }
  for (int i = 0; i <= heightLines; i++) {
    float posY = map(i, 0, heightLines, -height / 2, height / 2);
    // Lines parallel to X-axis
    vertex(-width / 2, posY,  -depth / 2);
    vertex(-width / 2, posY, depth / 2);
  }
  
  endShape();
}

void setGui() {
  PFont fontSmall = createFont("Century Gothic", 14, true);
  PFont fontLarge = createFont("Century Gothic Bold", 30, true);
  float X = width; //x position for gui elements
  float distance = 50; //vertical spacing between gui elements
      
  cp5.addSlider("boxWidth")
    .setPosition(X/2 -900, 850)
    .setFont(fontSmall)
    .setWidth(200)
    .setHeight(50)
    .setRange(1600, 16000)
    .setValue(boxWidth)
    .setColorActive(color(170,75,85))//active
    .setColorForeground(color(170,75,85)) 
    .setColorBackground(color(50)) //not active
    .plugTo(this,"boxWidth")
    .setLabel("Expand");
    
    
  cp5.addToggle("ShowBox")
    .setSize(50,20)
    .setValue(showBox)
    .setPosition(X/2 - 900, 1000)
    .setFont(fontSmall)     
    .plugTo(this, "showBox")    
    .setColorActive(color(255))//active
    .setColorForeground(color(255))
    .setColorBackground(color(50)) //not active
    .setLabel("Box"); 
    
 cp5.addToggle("ShowGrid")
        .setSize(50, 20)
        .setValue(showGrid)
        .setPosition(X/2 - 900, 925)
        .setFont(fontSmall)
        .setColorActive(color(255))//active
        .setColorForeground(color(255))
        .setColorBackground(color(50)) //not active        
        .plugTo(this, "showGrid")
        .setLabel("Grid");

    cp5.addToggle("ShowSet1")
        .setSize(50, 20)
        .setValue(showSet1)
        .setPosition(X/2 - 800, 925)
        .setFont(fontSmall)
        .setColorActive(color(140,90,210))//active
        .setColorForeground(color(140,90,210)) 
        .setColorBackground(color(50)) //not active
        .plugTo(this, "showSet1")
        .setLabel("Traditional Crafts / Textile Art");

    cp5.addToggle("ShowSet2")
        .setSize(50, 20)
        .setValue(showSet2)
        .setPosition(X/2 - 800, 1000) 
        .setFont(fontSmall)
        .setColorActive(color(180,215,205))//not active
        .setColorForeground(color(180,215,205))
        .setColorBackground(color(50)) // active
        .plugTo(this, "showSet2")
        .setLabel("Artificial Intelligence / Machine Learning");
        
  cp5.addButton("Show Items and Titles")
   .setPosition(X/2 - 900, 800) 
   .setSize(200, 20)
   .setFont(fontSmall)
   .setColorActive(color(50))//active
   .setColorForeground(color(155)) // on click
   .setColorBackground(color(50)) //not active
   .setLabel("Points")
   .onClick(new CallbackListener() {
     public void controlEvent(CallbackEvent event) {
       showItemsTitles = !showItemsTitles; 
     }
   });
   
  cp5.addButton("Show Curves")
   .setPosition(X/2 - 900, 775) // Adjust position as needed
   .setSize(200, 20)
   .setFont(fontSmall)
   .setColorActive(color(50))//active
   .setColorForeground(color(155)) // on click
   .setColorBackground(color(50)) //not active
   .setLabel("Curves")
   .onClick(new CallbackListener() {
     public void controlEvent(CallbackEvent event) {
       showCurves = !showCurves; // Toggle visibility
     }
   });
      
  cp5.addTextlabel("dynamicType")
  .setFont(new ControlFont(fontSmall))
  .setValue("")
  .setColorValue(color(255,255,255))
  .setPosition(X/2 - 500, distance);
  
  cp5.addTextlabel("dynamicDate")
  .setFont(new ControlFont(fontSmall))
  .setValue("")
  .setColorValue(color(255,255,255))
  .setPosition(X/2 - 500, distance + 40);
    
  cp5.addTextlabel("dynamicTitle")
  .setFont(new ControlFont(fontSmall))
  .setText("")
  .setColorValue(color(255,255,255))
  .setPosition(X/2 - 500, distance +20);
  
  
  cp5.addTextlabel("mainTitle")
  .setFont(new ControlFont(fontLarge))
  .setText("InterWeaving")
  .setColorValue(color(255,255,255))
  .setPosition( X/2 -900 , distance);
      
  cp5.setAutoDraw(false);
}

void gui(){
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD(); 
  
  float rectWidth = width; // Width Backgroud Rect
  float rectHeight = 140;  // Height Background Rect
  float rectX = width / 2 - rectWidth / 2; // Center the rectangle
  float rectY = 0; 
  fill(0); 
  noStroke();
  rect(rectX, rectY, rectWidth, rectHeight);
  
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}

void drawLabels() {
  fill(255);
  textSize(400);
    
  pushMatrix();
  textAlign(LEFT, TOP);
  translate(-boxWidth/2, boxHeight/2, boxDepth/2+10);
  scale(0.1);
  text(startYear, 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(CENTER, TOP);
  translate(0, boxHeight/2, boxDepth/2);
  scale(0.1);
  text("2022", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(boxWidth/2, boxHeight/2, boxDepth/2+10);
  scale(0.1);
  text(endYear, 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(-boxWidth/2 -10, boxHeight/2-50, boxDepth/2);
  scale(0.1);
  text("0", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(LEFT, BOTTOM);
  translate(-boxWidth/2 -10, 200, boxDepth/2);
  rotateZ(-HALF_PI);
  //translate(0, -boxHeight/2,0);
  scale(0.1);
  text("Check-out Duration (Hours)", 0, 0, 0);
  popMatrix(); 
  
  pushMatrix();
  textAlign(RIGHT, BOTTOM);
  translate(-boxWidth/2, -boxHeight/2, boxDepth/2);
  scale(0.1);
  text(maxCheckoutDuration(), 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(boxWidth/2+100, -boxHeight/2-50, boxDepth/2);
  scale(0.1);
  text("8AM", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(boxWidth/2 +100, -boxHeight /2 -50, -150);
  rotateY(HALF_PI);
  scale(0.1);
  text("Check-out Time", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(boxWidth/2 +100, boxHeight /2 -50, -150);
  rotateY(HALF_PI);
  scale(0.1);
  text("Check-out Date", 0, 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(RIGHT, TOP);
  translate(boxWidth/2+100, -boxHeight/2-50, -boxDepth/2);
  scale(0.1);
  text("8PM", 0, 0, 0);
  popMatrix();
}
